/******************************************************************************
 * sun_riset_main.c
 * Federico Bareilles
 * 07/05/2004
 *
 * Based on SUNRISET.C (c) Paul Schlyter, 1989, 1992
 *
 *****************************************************************************/
#define VERSION "0.1-d"

#define _POSIX_SOURCE 1

#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include <string.h>
#include <time.h>
//#include <math.h>
#include "sun_riset.h"
#include "ang_tools.h"

#define LATSET 0x01
#define LONSET 0x02
#define TZONE  0x04
#define HMSOUT 0x08

extern long int timezone;
extern char *tzname[2];
char *program_name;
char *program_version;


char *help_text[] ={
  "  -a, --latitude\t- Latitude in degrees (- is south, + is north).",
  "  -o, --longitude\t- Longitude in degrees (- is west, + is east).",
  "  [-y, --year]\t\t- Year from 1801 to 2099. Def. current year.",
  "  [-m, --month]\t\t- Month. Def. current month.",
  "  [-d, --day]\t\t- day. Def. current day.",
  "  [-t, --timezone]\t- Time zone. if none, use systeme TZ. (0 for UTC).",
  "  [--civil]\t\t- Civil twilight starts/ends when the Sun's\n\
\t\t\t  center is 6 degrees below the horizon. Default.",
  "  [--astro]\t\t- Astronomical twilight starts/ends when the Sun's\n\
\t\t\t  center is 18 degrees below the horizon.\n\t\t\t  Def: Civil twilight.",
  "  [--nautic]\t\t- Nautical twilight starts/ends when the Sun's\n\
\t\t\t  center is 12 degrees below the horizon.\n\t\t\t  Def: Civil twilight.",
  "  [-h]\t\t\t- Output in hh:mm:ss.",
  "  [--help]\t\t- this help !!!.",
  ""
};


void usage()
{
  fprintf(stderr," - Sun rise/set (v%s) -\n\
(c) Copyright 2004, 2015 Federico Bareilles <fede@libertad.fcaglp.unlp.edu.ar>\n\
Usage: \t%s -a/--latitude lat -o/--longitude long [-y/--year YYYY][-m/--month MM][-d/--day DD][-t/--timezone TZ][[--civil][--astro][--nautic]][--help]\n\
",
	  program_version, program_name);
}


void help()
{
  int i=0;

  while((char) *help_text[i] != 0)
    fprintf(stderr,"%s\n",help_text[i++]);
 fprintf(stderr,"\n");
}


int main(int argc, char *argv[])
{

  char *cp; /* for options */
  int year,month,day;
  double lon = 0.0 , 
    lat = 0.0;
  double rise, set;
  double astr_start, astr_end;
  double nautic_start, nautic_end;
  double tzone;
  int    rs, astr, nautic;
  int data = 0,
	  astro_flag = 0,
	  civil_flag = 0,
	  nautic_flag = 0;
  
  char hr[16], hs[16];

  int op;
  static struct option  const  longopts[] =
  {
    {"help", 0, 0, 101},
    {"latitude", 1, 0, 'a'},
    {"longitude", 1, 0, 'o'},
    {"year", 1, 0, 'y'},
    {"month", 1, 0, 'm'},
    {"day", 1, 0, 'd'},
    {"timezone", 1, 0, 't'},
    {"astro", 0, 0, 202},
    {"civil", 0, 0, 203},
    {"nautic", 0, 0, 204},
    {0,  0,  0,  0}
  };
  time_t t = time(NULL);
  struct tm *tms = localtime(&t);

  tzset(); /* initialize time conversion information */
  tzone = (double) - timezone / 3600.0;
  day = tms->tm_mday;
  month = tms->tm_mon + 1;
  year = tms->tm_year + 1900;

  /* who is my name? */
  if ((cp = strrchr(argv[0], '/')) != NULL) program_name  = cp + 1;
  else program_name  = argv[0];
  program_version = VERSION;
  opterr =  0;
  while  ((op = getopt_long(argc, argv, "a:o:y:m:d:t:h",longopts, 0)) != EOF)
    switch( op ){
    case 'a':
	    //lat=atof(optarg);
      lat=strdeg2double(optarg);
      data |= LATSET;
      break;
    case 'o':
	    //lon=atof(optarg);
      lon=strdeg2double(optarg);
      data |=LONSET;
      break;
    case 'y':
      year=atoi(optarg);
      break;
    case 'm':
      month=atoi(optarg);
      break;
    case 'd':
      day=atoi(optarg);
      break;
    case 't':
      tzone=atof(optarg);
      data |= TZONE;
      break;
    case 'h':
      data |= HMSOUT;
      break;
    case 202:
      astro_flag = 1;
      break;
    case 203:
      civil_flag = 1;
      break;
    case 204:
      nautic_flag = 1;
      break;
    case 101:
      usage();
      help();
      exit(-1);
    default:
      usage();
      exit(-1);
    }
  if( ! (data & LATSET && data & LONSET) ){
    usage();
    fprintf(stderr,"Latitude or longitude not defined.\n"); 
    exit(-1);
  }
  
  printf( "# SUN\n");
  printf( "# Lat: %7.3f, Lon: %7.3f\n", lat, lon);
  printf( "# Date: %4d/%02d/%02d\n", year, month, day);
  printf( "# Time zone: %2.0f", tzone);
  if( ! (data & TZONE) ) printf( " (%s)", tzname[0]);
  printf( "\n");

  
  if ( ! astro_flag && ! nautic_flag ) civil_flag = 1;
    
  if( astro_flag) {/* astro */
	  printf( "# astronomical twilight:\n");
	  astr = astronomical_twilight( year, month, day, lon, lat,
					&astr_start, &astr_end );
	  switch( astr ){
	  case 0:
		  printf( "# starts ends\n");
		  if (data & HMSOUT) {
			  double2gms_str(hr, astr_start + tzone);
			  double2gms_str(hs, astr_end + tzone);
			  printf( "%s %s\n", hr , hs);
		  }else
			  printf( "%5.2f %5.2f\n",
				  astr_start + tzone , astr_end + tzone);
		  break;
	  case +1:
		  printf( "# Never darker than astronomical twilight\n" );
		  break;
	  case -1:
		  printf( "# Never as bright as astronomical twilight\n" );
		  break;
	  }
  }
  if( nautic_flag) {/* nautical */
	  printf( "# nautical twilight:\n");
	  nautic = nautical_twilight( year, month, day, lon, lat,
					&nautic_start, &nautic_end );
	  switch( nautic ){
	  case 0:
		  printf( "# starts ends\n");
		  if (data & HMSOUT) {
			  double2gms_str(hr, nautic_start + tzone);
			  double2gms_str(hs, nautic_end + tzone);
			  printf( "%s %s\n", hr , hs);
		  }else
			  printf( "%5.2f %5.2f\n",
				  nautic_start + tzone , nautic_end + tzone);
		  break;
	  case +1:
		  printf( "# Never darker than nautical twilight\n" );
		  break;
	  case -1:
		  printf( "# Never as bright as nautical twilight\n" );
		  break;
	  }
  }
  if (civil_flag) { /* Civil: */
	  printf( "# civil twilight:\n");
	  rs = sun_rise_set( year, month, day, lon, lat, &rise, &set );
	  switch( rs ){
	  case 0:
		  printf( "# rises sets\n");
		  if (data & HMSOUT) {
			  double2gms_str(hr, rise + tzone);
			  double2gms_str(hs, set + tzone);
			  printf( "%s %s\n", hr , hs);
		  }else
			  printf( "%5.2f %5.2f\n",
				  rise + tzone , set + tzone);
		  break;
	  case +1:
		  printf( "# Sun above horizon\n" );
		  break;
	  case -1:
		  printf( "# Sun below horizon\n" );
		  break;
	  }
  }
  
  return 0;
}
